@@ -47,10 +47,6 @@
 
 #include <errno.h>
 
-#include <gobject/gobject.h>
-#include <gobject/gparamspecs.h>
-#include <gobject/gvaluetypes.h>
-
 #include <bonobo-activation/bonobo-activation.h>
 #include <bonobo-activation/bonobo-activation-version.h>
 #include <libbonobo.h>
@@ -67,7 +63,7 @@ bonobo_post_args_parse (GnomeProgram *program, GnomeModuleInfo *mod_info)
 	int dumb_argc = 1;
 	char *dumb_argv[] = {NULL};
 
-	dumb_argv[0] = program_invocation_name;
+	dumb_argv[0] = g_get_prgname ();
 
 	bonobo_init (&dumb_argc, dumb_argv);
 }
@@ -92,7 +88,7 @@ gnome_bonobo_module_info_get (void)
 		N_("Bonobo Support"),
 		NULL, NULL,
 		NULL, bonobo_post_args_parse,
-		NULL, NULL, NULL
+		NULL, NULL, NULL, NULL
 	};
 
 	if (module_info.requirements == NULL) {
@@ -120,19 +116,22 @@ bonobo_activation_pre_args_parse (GnomeProgram *program, GnomeModuleInfo *mod_in
         if (!g_thread_supported ())
 		g_thread_init (NULL);
 
-	bonobo_activation_preinit (program, mod_info);
+	if (!bonobo_activation_is_initialized ())
+		bonobo_activation_preinit (program, mod_info);
 }
 
 static void
 bonobo_activation_post_args_parse (GnomeProgram *program, GnomeModuleInfo *mod_info)
 {
-	int dumb_argc = 1;
-	char *dumb_argv[] = {NULL};
+	if (!bonobo_activation_is_initialized ()) {
+		int dumb_argc = 1;
+		char *dumb_argv[] = {NULL};
 
-	dumb_argv[0] = program_invocation_name;
-	(void) bonobo_activation_orb_init (&dumb_argc, dumb_argv);
-
-	bonobo_activation_postinit (program, mod_info);
+		dumb_argv[0] = g_get_prgname ();
+		(void) bonobo_activation_orb_init (&dumb_argc, dumb_argv);
+		
+		bonobo_activation_postinit (program, mod_info);
+	}
 }
 
 /* No need to make this public, always pulled in */
@@ -168,8 +167,10 @@ static char *gnome_user_accels_dir = NULL;
 /**
 * gnome_user_dir_get:
 *
-* Returns: An absolute path to the user-specific directory for GNOME apps to
-* use.
+* Retrieves the user-specific directory for GNOME apps to use ($HOME/.gnome2
+* is the usual GNOME 2 value).
+*
+* Returns: An absolute path to the directory.
 */
 const char *
 gnome_user_dir_get (void)
@@ -197,8 +198,12 @@ gnome_user_private_dir_get (void)
 /**
 * gnome_user_accels_dir_get:
 *
-* Returns: The absolute path to the user-specific directory that stores the
-* keyboard shortcut files for each GNOME app.
+* Retrieves the user-specific directory that stores the keyboard shortcut files
+* for each GNOME app. Note that most applications should be using GConf for
+* storing this information, but it may be necessary to use the
+* gnome_user_accels_dir_get() directory for legacy applications.
+*
+* Returns: The absolute path to the directory.
 */
 const char *
 gnome_user_accels_dir_get (void)
@@ -222,24 +227,24 @@ libgnome_option_cb (poptContext ctx, enum poptCallbackReason reason,
 		case ARG_ESPEAKER:
 			g_value_init (&value, G_TYPE_STRING);
 			g_value_set_string (&value, opt->arg);
-			g_object_set (G_OBJECT (program),
-				      GNOME_PARAM_ESPEAKER, &value);
+			g_object_set_property (G_OBJECT (program),
+					       GNOME_PARAM_ESPEAKER, &value);
 			g_value_unset (&value);
 			break;
 
 		case ARG_DISABLE_SOUND:
 			g_value_init (&value, G_TYPE_BOOLEAN);
 			g_value_set_boolean (&value, FALSE);
-			g_object_set (G_OBJECT (program),
-				      GNOME_PARAM_ENABLE_SOUND, &value);
+			g_object_set_property (G_OBJECT (program),
+					       GNOME_PARAM_ENABLE_SOUND, &value);
 			g_value_unset (&value);
 			break;
 
 		case ARG_ENABLE_SOUND:
 			g_value_init (&value, G_TYPE_BOOLEAN);
 			g_value_set_boolean (&value, TRUE);
-			g_object_set (G_OBJECT (program),
-				      GNOME_PARAM_ENABLE_SOUND, &value);
+			g_object_set_property (G_OBJECT (program),
+					       GNOME_PARAM_ENABLE_SOUND, &value);
 			g_value_unset (&value);
 			break;
 
@@ -260,9 +265,33 @@ static void
 libgnome_userdir_setup (gboolean create_dirs)
 {
 	if(!gnome_user_dir) {
-		gnome_user_dir = g_build_filename (g_get_home_dir(), ".gnome", NULL);
-		gnome_user_private_dir = g_build_filename (g_get_home_dir(),
-							   ".gnome_private", NULL);
+                const char *override;
+
+                /* FIXME this env variable should be changed
+                 * for each major GNOME release, would be easier to
+                 * remember if not hardcoded.
+                 */
+                override = g_getenv ("GNOME22_USER_DIR");
+                
+                if (override != NULL) {
+                        int len;
+
+                        gnome_user_dir = g_strdup (override);
+
+                        /* chop trailing slash */
+                        len = strlen (gnome_user_dir);
+                        if (len > 1 && gnome_user_dir[len-1] == '/')
+                                gnome_user_dir[len - 1] = '\0';
+                        
+                        gnome_user_private_dir = g_strconcat (gnome_user_dir,
+                                                              "_private",
+                                                              NULL);
+                } else {
+                        gnome_user_dir = g_build_filename (g_get_home_dir(), GNOME_DOT_GNOME, NULL);
+                        gnome_user_private_dir = g_build_filename (g_get_home_dir(),
+                                                                   GNOME_DOT_GNOME_PRIVATE, NULL);
+                }
+                
 		gnome_user_accels_dir = g_build_filename (gnome_user_dir,
 							  "accels", NULL);
 	}
@@ -344,13 +373,10 @@ libgnome_post_args_parse (GnomeProgram *program,
 
 	libgnome_userdir_setup (create_dirs_val);
 
-	setlocale (LC_ALL, "");
-	/* XXX todo - handle multiple installation dirs */
-	bindtextdomain (PACKAGE, LIBGNOME_LOCALEDIR);
 }
 
 static struct poptOption gnomelib_options [] = {
-	{ NULL, '\0', POPT_ARG_INTL_DOMAIN, PACKAGE, 0, NULL, NULL},
+	{ NULL, '\0', POPT_ARG_INTL_DOMAIN, GETTEXT_PACKAGE, 0, NULL, NULL},
 
 	{ NULL, '\0', POPT_ARG_CALLBACK, (void *) libgnome_option_cb, 0, NULL, NULL},
 
@@ -365,9 +391,11 @@ static struct poptOption gnomelib_options [] = {
 				 " running"),
 	  N_("HOSTNAME:PORT")},                                                 
 
-	{"version", '\0', POPT_ARG_NONE, NULL, },
+	{"version", '\0', POPT_ARG_NONE,
+	 NULL, ARG_VERSION, VERSION, NULL},
 
-	{ NULL, '\0', 0, NULL, 0 }
+	{ NULL, '\0', 0,
+	  NULL, 0 , NULL, NULL}
 };
 
 static void
@@ -412,6 +440,11 @@ libgnome_module_info_get (void)
 	if (module_info.requirements == NULL) {
 		static GnomeModuleRequirement req[4];
 
+		bindtextdomain (GETTEXT_PACKAGE, LIBGNOME_LOCALEDIR);
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
+		bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+#endif
+
 		req[0].required_version = "0.9.1";
 		req[0].module_info = gnome_bonobo_activation_module_info_get ();
 
